local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )


return {
	name = STRINGS.IMPROVISATION.ACTIONS.INJECT_LETHAL,
	getName = function( self, sim, unit )
		return self.name
	end,
	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip(
			string.format(STRINGS.IMPROVISATION.ACTIONS.INJECT_LETHAL, target and target:getName() or ""),
			string.format(STRINGS.IMPROVISATION.ACTIONS.INJECT_LETHAL_TIP, abilityOwner:getName() or "item")
		)
	end,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_medigel_small.png",

	-- alwaysShow = true,
	proxy = true,

	-- per-unit validation
	isValidTarget = function( self, sim, unit, userUnit, targetUnit )
		if userUnit ~= unit and userUnit ~= unit:getUnitOwner()
			then return false end

		if targetUnit == nil or targetUnit:isGhost()
			then return false end

		if not simquery.isEnemyTarget( userUnit:getPlayerOwner(), targetUnit )
			then return false end
		
		if targetUnit:hasTrait("isDrone")
			then return false end
		
		-- if not targetUnit:hasTrait("canKO") or not targetUnit:isKO() or targetUnit:isDead()
		if targetUnit:isDead()
			then return false end

		local pinned, pinner = simquery.isUnitPinned(sim, targetUnit)
		if pinner ~= userUnit
			then return false end

		return true
	end,
	
	-- listing all potential targets
	findTargets = function( self, sim, unit, userUnit )
		userUnit = userUnit or unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )

		local units = {}
		
		for i,cellUnit in ipairs(cell.units) do
			if self:isValidTarget(sim, unit, userUnit, cellUnit) then
				table.insert( units, cellUnit )
			end
		end

		return units
	end,
	
	-- getting a specific target
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		userUnit = userUnit or unit:getUnitOwner()
		return targets.unitTarget( game, self:findTargets(sim, unit, userUnit) , self, unit, userUnit )
	end,
	
	-- target validation and item conditions
	canUseAbility = function( self, sim, unit, userUnit, targetID )

		-- Must have a user owner.
		userUnit = userUnit or unit:getUnitOwner()
		if not userUnit then
			return false
		end

		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end

		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end
	
		local ok, reason = abilityutil.checkRequirements( unit, userUnit )
		if not ok then
			return false, reason
		end

		if userUnit:getAP() < 1 then 
			return false, STRINGS.UI.REASON.ATTACK_USED
		end 
		
		
		if targetID then
			local targetUnit = sim:getUnit( targetID )
			
			if not self:isValidTarget(sim, unit, userUnit, targetUnit) then
				return false, STRINGS.UI.REASON.INVALID_TARGET
			end
			
			if unit:hasTrait("armorPiercing") then --if this doesn't specify 0 armorPiercing then ignore armor
				local meleeDmg,armorPiercing, armor = simquery.calculateDamageAndArmorForMelee(sim, unit, targetUnit)
				if (armor or 0) - armorPiercing > 0 then
					return false, STRINGS.UI.COMBAT_PANEL_ARMORED
				end
			end
			
			if unit:getTraits().armorPWRcost then
				local PWRcostPoints = math.max(unit:getTraits().armorPiercing - armorPiercing + armor,0)
				if PWRcostPoints and userUnit:getPlayerOwner():getCpus() < unit:getTraits().armorPWRcost * PWRcostPoints then
					return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
				end
			end

		else
			local units = self:findTargets( sim, unit, userUnit )
			if #units == 0 then
				return false, STRINGS.UI.REASON.NO_VIABLE_TARGET
			end
		end
		
		
		return true
	end,
	
	executeAbility = function( self, sim, unit, userUnit, target )
		if not unit:isValid() then
			return
		end
		
		local userUnit = unit:getUnitOwner()
		local targetUnit = sim:getUnit(target)	

		local x0,y0 = userUnit:getLocation()
		
		sim:dispatchEvent( simdefs.EV_UNIT_HEAL, { unit = userUnit, target = targetUnit, revive = false, facing = userUnit:getFacing() } )
		
		if unit:getTraits().armorPWRcost and targetUnit:getArmor() then
			local meleeDmg,armorPiercing, armor = simquery.calculateDamageAndArmorForMelee(sim, unit, targetUnit)
			local PWRcostPoints = math.max(unit:getTraits().armorPiercing - armorPiercing + armor,0)
			if PWRcostPoints > 0 then
				unit:getPlayerOwner():addCPUs( -(unit:getTraits().armorPWRcost * PWRcostPoints), sim, x0,y0)	
			end
		end
		
		targetUnit:killUnit(sim)
		
		if unit:hasTrait("disposable") then 
			inventory.trashItem( sim, userUnit, unit )
		else
			inventory.useItem( sim, userUnit, unit )
		end
		
		--sim:triggerEvent(simdefs.TRG_UNIT_HIT, {targetUnit=targetUnit, sourceUnit=userUnit, x=x0, y=y0, melee=true})
		
		userUnit:useAP( sim )
		
	end,
}